#include <FL/Fl.H>
#include <FL/fl_draw.H>
#include "game_state.hh"
#include "sun.hh"
#include "ship.hh"

extern double time_interval;

void 
draw_item(Vector_Object * object){
  object->draw();
}

void update_position(Vector_Object * object){
  object->update_position(time_interval);
}

void check_is_alive(Vector_Object * object, vector_list * del_list){
  if(!object->is_alive())
    del_list->add_element(object);
}

void accelerate_item(Vector_Object * object, Gravity_Object * gravity){
  gravity->accelerate_object(object,time_interval);
}

void 
calc_gravity(Gravity_Object * object,vector_list * v_list){
  v_list->for_each_with_ptr((void (*)(Vector_Object *, void *))&accelerate_item,
			     (void *)object);
}

struct Collide_Data {
  Ship * ship;
  bool do_delete;
};


void 
check_collide(Vector_Object * object, Collide_Data * data  ){
  Ship * ship = data->ship;
  if(((object->get_x()-ship->get_x())*(object->get_x()-ship->get_x())
     +(object->get_y()-ship->get_y())*(object->get_y()-ship->get_y()) 
     <  object->sqr_size()) && 
     (object != ship)){
    data->do_delete = true;
  }
}

void 
game_state::initialize(){
  Sun * sun_data, * sun2_data;
  //Ship * ship1, *ship2;
  sun_data = new Sun(320,215,-10,0,10000,10);
  sun2_data = new Sun(320,265,10,0,10000,10);
  right_ship = new Ship(540,240,0,10,FL_CYAN,this);
  left_ship = new Ship(100,240,0,-10,FL_GREEN,this);
  g_list.clear();
  v_list.clear_and_delete();
  v_list.add_element(sun_data);
  g_list.add_element(sun_data);
  v_list.add_element(sun2_data);
  g_list.add_element(sun2_data);
  v_list.add_element(right_ship);
  v_list.add_element(left_ship);
  right_player_score = 0;
  left_player_score = 0;
};

void 
game_state::update(){
  vector_list del_list;
  Collide_Data data;

  v_list.for_each_with_ptr((void (*)(Vector_Object *, void *))&check_is_alive,
			   (void *)&del_list);

  while(!del_list.is_empty()){
    v_list.remove(del_list.first_element());
  }

  data.ship = left_ship;
  data.do_delete = false;
  v_list.for_each_with_ptr((void (*)(Vector_Object *,void *))&check_collide,
			   (void *)&data);
  if(data.do_delete){
    delete left_ship;
    right_player_score++;
    left_ship = new Ship(100,240,0,-10,FL_GREEN,this);
  }

  data.ship = right_ship;
  data.do_delete = false;
  v_list.for_each_with_ptr((void (*)(Vector_Object *,void *))&check_collide,
			   (void *)&data);
  if(data.do_delete){
    delete right_ship;
    left_player_score++;
    right_ship = new Ship(540,240,0,10,FL_CYAN,this);
  }


  g_list.for_each_with_ptr((void (*)(Gravity_Object *, void *))&calc_gravity,
			   (void *)&v_list);
  v_list.for_each(&update_position);
};

void
game_state::update_screen(){
  char score[256];
  v_list.for_each(&(draw_item));
  sprintf(score,"%d",left_player_score);
  fl_draw(score,20,50);
  sprintf(score,"%d",right_player_score);
  fl_draw(score,610,50);
};

void 
game_state::add_missile(Missile * missile){
  v_list.add_element(missile);
}
