#ifndef VECTOR_OBJECT_HH
#define VECTOR_OBJECT_HH
#include <math.h>

extern double screen_width, screen_height;

class Vector_Object {
public:
  inline double get_x(){ return x;};
  inline double get_y(){ return y;};
  inline double get_dx(){return dx;};
  inline double get_dy(){return dy;};
  inline double get_angle(){return atan2(dy,dx);};
  inline double get_magnitude(){ return hypot(dy,dx);};
  inline void set_xy(double x,double y){ this->x = x; this->y = y;};
  inline void set_dxdy(double dx,double dy){ this->dx = dx; this->dy = dy;};
  inline void set_angle_magnitude(double angle,double magnitude){
    dx = cos(angle)*magnitude;
    dy = sin(angle)*magnitude;
  }

  inline void accelerate(double ddx, double ddy){
    dx += ddx;
    dy += ddy;
  }

  inline void update_position(double time){
    x += time*dx;
    x = (x/screen_width - floor(x/screen_width))*screen_width;
    y += time*dy;
    y = (y/screen_height - floor(y/screen_height))*screen_height;
  }

  Vector_Object(double x,double y, double dx, double dy){
    this->x = x;
    this->y = y;
    this->dx = dx;
    this->dy = dy;
  }

  virtual void draw() = 0;
  virtual bool is_alive() = 0;
  virtual double sqr_size() = 0;
private:
  double x,y;
  double dx, dy;
};

#endif // VECTOR_OBJECT_HH
